// R3DCamera.cpp: implementation of the R3DCamera class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DCamera.h"
#include "R3DCalc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// Globals:
R3DCamera *R3DCAM;	// a pointer to the current camera

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DCamera::R3DCamera()
{
	mWorldPos.SetPosition(320.f, 240.f, -640.f);	// default camera position
	mPan=mTilt=0.f;
	mTransformed=TRUE;
}

R3DCamera::~R3DCamera()
{

}

// These transformations cam be applied to the camera:
void R3DCamera::Pan(float angle)
{
	mPan+=angle;
	mTransformed=TRUE;
}

void R3DCamera::Tilt(float angle)
{
	mTilt+=angle;
	mTransformed=TRUE;
}

void R3DCamera::Move(R3DVector v)
{
	mWorldPos+=v;
	mTransformed=TRUE;
}

// This takes a vector and transforms it from world co-ordinates
// to camera co-ordinates.
R3DVector R3DCamera::VProject(R3DVector v)
{
	if(mTilt!=0)
		v.RotateX(mWorldPos, -mTilt);	// rotate according to tilt
	if(mPan!=0)
		v.RotateY(mWorldPos, -mPan);	// rotate according to pan
	v-=mWorldPos;				// move whole world accordingly
	return v;
}

// this draws all the triangles that have been painted to the camera:
void R3DCamera::Draw()
{
	mTransformed=FALSE;	// for now, just set the flag to false so
						// that if the camera doesnt move before the
						// next frame we know not to bother with the
						// world->camera transformations
}


// Moves the camera forward or back in the direction it is facing without
// moving it up or down
void R3DCamera::Walk(float distance)
{
	float x=distance*R3DCALC->Sin(-mPan);
	float z=distance*R3DCALC->Cos(-mPan);
	mWorldPos+=R3DVector(x, 0.f, z);	// move the calculated distance
	mTransformed=TRUE;
}
