// R3DCuboid.cpp: implementation of the R3DCuboid class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DCuboid.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DCuboid::R3DCuboid()
{

}

R3DCuboid::~R3DCuboid()
{

}

// Draws the cube
void R3DCuboid::Draw()
{
	// position all the rectangles:
	SetupFaces();

	// actually draw the rectangles:
	for(int x=0; x<=5; x++)
		mFaces[x].Draw();
}


// generates cuboid by interpolating other 6 vectors orthogonally
void R3DCuboid::SetPosition(R3DVector v1, R3DVector v2)
{
	// v1 = top-left-nearest
	// v2 = bottom-right-farthest
	mCorners[0]=v1;
	mCorners[1].SetPosition(v2.m_vx, v1.m_vy, v1.m_vz);
	mCorners[2].SetPosition(v1.m_vx, v2.m_vy, v1.m_vz);
	mCorners[3].SetPosition(v2.m_vx, v2.m_vy, v1.m_vz);

	mCorners[4].SetPosition(v1.m_vx, v1.m_vy, v2.m_vz);
	mCorners[5].SetPosition(v2.m_vx, v1.m_vy, v2.m_vz);
	mCorners[6].SetPosition(v1.m_vx, v2.m_vy, v2.m_vz);
	mCorners[7]=v2;
}



// rotate the cuboid about the X axis
void R3DCuboid::RotateX(R3DVector Origin, float Theta)
{
	for(int x=0; x<=7; x++)
		mCorners[x].RotateX(Origin, Theta);
}

// rotate the cuboid about the Y axis
void R3DCuboid::RotateY(R3DVector Origin, float Theta)
{
	for(int x=0; x<=7; x++)
		mCorners[x].RotateY(Origin, Theta);
}

// rotate the cuboid about the Z axis
void R3DCuboid::RotateZ(R3DVector Origin, float Theta)
{
	for(int x=0; x<=7; x++)
		mCorners[x].RotateZ(Origin, Theta);
}

void R3DCuboid::SetTexture(R3DTexture t)
{
	SetupFaces();
	for(int x=0; x<=5; x++)
		mFaces[x].SetTexture(t);	// sets texture for each of the 6 rectangles
}


// moves the cube relative by the specified
void R3DCuboid::Move(R3DVector v)
{
	for(int x=0; x<=7; x++)
		mCorners[x]+=v;
}


// arranges the six rectangles according to the eight corners
/* corners go like this:

       4----------5
	  /|         /|
	 / |        / |
    0----------1  |
	|  |	   |  |
	|  6-------|--7
	| /		   | /
	|/		   |/
	2----------3
*/
void R3DCuboid::SetupFaces()
{
	mFaces[0].SetPosition(mCorners[0], mCorners[1], mCorners[2], mCorners[3]);	// front
	mFaces[1].SetPosition(mCorners[1], mCorners[5], mCorners[3], mCorners[7]);	// right
	mFaces[2].SetPosition(mCorners[5], mCorners[4], mCorners[7], mCorners[6]);	// back
	mFaces[3].SetPosition(mCorners[4], mCorners[0], mCorners[6], mCorners[2]);	// left
	mFaces[4].SetPosition(mCorners[4], mCorners[5], mCorners[0], mCorners[1]);	// top
	mFaces[5].SetPosition(mCorners[2], mCorners[3], mCorners[6], mCorners[7]);	// bottom
}

// serialize a cuboid
IMPLEMENT_SERIAL( R3DCuboid, CObject, 1 )
void R3DCuboid::Serialize(CArchive & archive)
{
	GrMipMapId_t t;

	if(archive.IsStoring())
		{
		t=mTexture.GetTextureId();
		archive << t;
		}
	else
		{
		archive >> t;
		mTexture=R3DTexture(t);	// construct texture based on saved ID
		}
	
	for(int x=0; x<=7; x++)
		mCorners[x].Serialize(archive);

	for(x=0; x<=5; x++)
		mFaces[x].Serialize(archive);
}

// Copy constructor, needed because CObject declares a private one
// that just causes me a big headache!
R3DCuboid::R3DCuboid(const R3DCuboid & c)
{
	for(int x=0; x<=7; x++)
		mCorners[x]=c.mCorners[x];
	for(x=0; x<=5; x++)
		mFaces[x]=c.mFaces[x];
	mOrigin=c.mOrigin;
	mTexture=c.mTexture;
}

// assignment operator
R3DCuboid& R3DCuboid::operator =(const R3DCuboid & c)
{
	for(int x=0; x<=7; x++)
		mCorners[x]=c.mCorners[x];
	for(x=0; x<=5; x++)
		mFaces[x]=c.mFaces[x];
	mOrigin=c.mOrigin;
	mTexture=c.mTexture;
	return *this;
}
