// R3DCylinder.cpp: implementation of the R3DCylinder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DCylinder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DCylinder::R3DCylinder()
{
	mRadius=0.f;
}

R3DCylinder::~R3DCylinder()
{

}

// initialises the position of all the corners
void R3DCylinder::SetPosition(R3DVector centre, float depth, float radius)
{
	float front=centre.m_vz-(depth/2);
	float back=centre.m_vz+(depth/2);
	R3DVector temp;

	mCentre=centre;
	mRadius=radius;
	mCircumference=radius*6.283185307f;

	mCorners[0]=centre;
	mCorners[0].m_vz=front;	// front centre
	mCorners[1]=centre;
	mCorners[1].m_vz=back;	// back centre

	temp=centre;
	temp.m_vy-=radius;	// move up to the outside
		
	for(int x=0; x<=9; x++)
		{
		int xf=(2*x)+2;	// index of front corner
		int xb=(2*x)+3;	// index of back corner

		mCorners[xf]=temp;
		mCorners[xf].m_vz=front;
		mCorners[xb]=temp;
		mCorners[xb].m_vz=back;

		temp.RotateZ(centre, 36);	// rotate to get next point in circle
		}
}

// positions the triangles and rectangles based on the corners
void R3DCylinder::SetupFaces()
{
	for(int x=0; x<=9; x++)
		{
		int xf=2*x;	// index for front triangles
		int xb=xf+1;// index for back triangles

		int e1f=(2*x)+2;
		int e1b=(2*x)+3;	// first edge front and back
		int e2f=e1f+2;
		int e2b=e1b+2;		// second edge front and back

		if (e2f==22)
			{
			e2f=2;	// wrap around for final corner
			e2b=3;
			}
		
		mTriangles[xf].SetPosition(mCorners[0], mCorners[e1f], mCorners[e2f]);
		mTriangles[xb].SetPosition(mCorners[1], mCorners[e1b], mCorners[e2b]);
		mRectangles[x].SetPosition(mCorners[e1f], mCorners[e1b], mCorners[e2f], mCorners[e2b]);
		}
}

// set the two textures
void R3DCylinder::SetTexture(R3DTexture face, R3DTexture edge)
{
	SetupFaces();	// positions all shapes as their positions affect the textures
	mFaceTexture=face;
	mEdgeTexture=edge;
	for(int x=0; x<=9; x++)
		{
		mTriangles[2*x].SetupTexture(0.f, 0.f, mFaceTexScale);
		mTriangles[(2*x)+1].SetupTexture(0.f, 0.f, mFaceTexScale);
		mRectangles[x].SetupTexture(0.f, 0.f, mEdgeTexScale);
		mTriangles[2*x].SetTexture(face);
		mTriangles[(2*x)+1].SetTexture(face);
		mRectangles[x].SetTexture(edge);
		}
}


// moving and rotating stuff:
void R3DCylinder::Move(R3DVector v)
{
	for(int x=0; x<=21; x++)
		mCorners[x]+=v;
	mCentre+=v;			// track the centre
}


void R3DCylinder::RotateX(R3DVector Origin, float Theta)
{
	for(int x=0; x<=21; x++)
		mCorners[x].RotateX(Origin, Theta);

	mRotation+=R3DVector(Theta, 0.f, 0.f);	// track total rotation
	if(mRotation.m_vx>=360.f)
		mRotation.m_vx-=360.f;
	if(mRotation.m_vx<0.f)
		mRotation.m_vx+=360.f;	// correct so angle stays between 0 & 359
	mCentre.RotateX(Origin, Theta);			// track the centre
}

void R3DCylinder::RotateY(R3DVector Origin, float Theta)
{
	for(int x=0; x<=21; x++)
		mCorners[x].RotateY(Origin, Theta);

	mRotation+=R3DVector(0.f, Theta, 0.f);	// track total rotation
	if(mRotation.m_vy>=360.f)
		mRotation.m_vy-=360.f;
	if(mRotation.m_vy<0.f)
		mRotation.m_vy+=360.f;	// correct so angle stays between 0 & 359
	mCentre.RotateY(Origin, Theta);			// track the centre
}

void R3DCylinder::RotateZ(R3DVector Origin, float Theta)
{
	for(int x=0; x<=21; x++)
		mCorners[x].RotateZ(Origin, Theta);

	mRotation+=R3DVector(0.f, 0.f, Theta);	// track the rotation
	if(mRotation.m_vy>=360.f)
		mRotation.m_vy-=360.f;
	if(mRotation.m_vy<0.f)
		mRotation.m_vy+=360.f;	// correct so angle stays between 0 & 359
	mCentre.RotateZ(Origin, Theta);			// track the centre
}


///////////////////////////////////////////////////////////////////
// actually draw the cylinder!
void R3DCylinder::Draw()
{
	SetupFaces();	// positions all shapes before drawing
	for(int x=0; x<=9; x++)
		{
		mTriangles[2*x].Draw();
		mTriangles[(2*x)+1].Draw();
		mRectangles[x].Draw();
		}
}

void R3DCylinder::SetTexture(R3DTexture t)
{
	this->SetTexture(t, t);
}

// set the texture scales for the triangles and edges
void R3DCylinder::SetupTexture(float facescale, float edgescale)
{
	if(0!=facescale)
		mFaceTexScale=facescale;
	if(0!=edgescale)
		mEdgeTexScale=edgescale;
}

// serialize a cylinder
IMPLEMENT_SERIAL( R3DCylinder, CObject, 1 )
void R3DCylinder::Serialize(CArchive & archive)
{
	GrMipMapId_t t1, t2;
	if(archive.IsStoring())
		{
		t1=mEdgeTexture.GetTextureId();
		t2=mFaceTexture.GetTextureId();
		archive << mEdgeTexScale << t1 << mFaceTexScale << t2;
		archive << mCircumference << mRadius;
		}
	else
		{
		archive >> mEdgeTexScale >> t1 >> mFaceTexScale >> t2;
		mEdgeTexture=R3DTexture(t1);
		mFaceTexture=R3DTexture(t2);
		archive >> mCircumference >> mRadius;
		}
	
	for(int x=0; x<=21; x++)
		mCorners[x].Serialize(archive);
	for(x=0; x<=9; x++)
		mRectangles[x].Serialize(archive);
	for(x=0; x<=19; x++)
		mTriangles[x].Serialize(archive);
	mCentre.Serialize(archive);
	mRotation.Serialize(archive);
}

// copy constructor
R3DCylinder::R3DCylinder(const R3DCylinder &c)
{
	for(int x=0; x<=21; x++)
		mCorners[x]=c.mCorners[x];
	for(x=0; x<=9; x++)
		mRectangles[x]=c.mRectangles[x];
	for(x=0; x<=19; x++)
		mTriangles[x]=c.mTriangles[x];
	mEdgeTexScale=c.mEdgeTexScale;
	mFaceTexScale=c.mFaceTexScale;
	mEdgeTexture=c.mEdgeTexture;
	mFaceTexture=c.mFaceTexture;
}

// assignment operator
R3DCylinder& R3DCylinder::operator =(const R3DCylinder & c)
{
	for(int x=0; x<=21; x++)
		mCorners[x]=c.mCorners[x];
	for(x=0; x<=9; x++)
		mRectangles[x]=c.mRectangles[x];
	for(x=0; x<=19; x++)
		mTriangles[x]=c.mTriangles[x];
	mEdgeTexScale=c.mEdgeTexScale;
	mFaceTexScale=c.mFaceTexScale;
	mEdgeTexture=c.mEdgeTexture;
	mFaceTexture=c.mFaceTexture;
	return *this;
}

// Spins a cylinder about its axle
void R3DCylinder::Spin(float distance)
{
	float theta=(360*distance)/mCircumference;	// angle to spin by
	
	for(int x=0; x<=21; x++)
		{
		// un-rotate the cylinder so it's on its original axis:
		mCorners[x].RotateX(mCentre, -mRotation.m_vx);
		mCorners[x].RotateY(mCentre, -mRotation.m_vy);
		mCorners[x].RotateZ(mCentre, -mRotation.m_vz);
		// rotate according to the spin angle:
		mCorners[x].RotateZ(mCentre, theta);
		// re rotate the cylinder back how it was:
		mCorners[x].RotateX(mCentre, mRotation.m_vx);
		mCorners[x].RotateY(mCentre, mRotation.m_vy);
		mCorners[x].RotateZ(mCentre, mRotation.m_vz);
		}
}
