// R3DModel.h: interface for the R3DModel class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_R3DMODEL_H__322300E3_08C5_11D2_9F89_0000E822356F__INCLUDED_)
#define AFX_R3DMODEL_H__322300E3_08C5_11D2_9F89_0000E822356F__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <afxtempl.h>
#include "R3DShape.h"
#include "R3DTriangle.h"
#include "R3DRectangle.h"
#include "R3DCuboid.h"
#include "R3DCylinder.h"

class R3DModel : public CObject, R3DShape  
{
public:
	DECLARE_SERIAL( R3DModel )	// models can be serialised
	void Serialize( CArchive& archive );
	void SetTexture(R3DTexture t);
	void Draw();
	void Move(R3DVector v);
	void RotateZ(R3DVector Origin, float Theta);
	void RotateY(R3DVector Origin, float Theta);
	void RotateX(R3DVector Origin, float Theta);
	void AddCylinder(R3DCylinder& c);
	void AddCuboid(R3DCuboid &c);
	void AddRectangle(R3DRectangle &r, int v1, int v2, int v3, int v4);
	void AddTriangle(R3DTriangle &t, int v1, int v2, int v3);
	void SetupVectors(R3DVector *v, int number);
	R3DModel (int ntriangles, int nrectangles, int ncuboids, int ncylinders);
	R3DModel();
	virtual ~R3DModel();

protected:
	CArray<R3DVector, R3DVector&> mVectors;
	CArray<R3DTriangle, R3DTriangle&> mTriangles;
	CArray<int, int> mTriangleMap;
	CArray<R3DRectangle, R3DRectangle&> mRectangles;
	CArray<int, int> mRectangleMap;
	CArray<R3DCuboid, R3DCuboid&> mCuboids;
	CArray<R3DCylinder, R3DCylinder&> mCylinders;

	// actual number not necessarily the size of the arrays
	int mNumVectors;
	int mNumTriangles;
	int mNumRectangles;
	int mNumCuboids;
	int mNumCylinders;
};

// related functions:
template <> void AFXAPI SerializeElements <R3DTriangle> ( CArchive& ar, R3DTriangle* t, int nCount );
template <> void AFXAPI SerializeElements <R3DRectangle> ( CArchive& ar, R3DRectangle* t, int nCount );
template <> void AFXAPI SerializeElements <R3DCuboid> ( CArchive& ar, R3DCuboid* t, int nCount );
template <> void AFXAPI SerializeElements <R3DCylinder> ( CArchive& ar, R3DCylinder* t, int nCount );

#endif // !defined(AFX_R3DMODEL_H__322300E3_08C5_11D2_9F89_0000E822356F__INCLUDED_)
