// R3DRectangle.cpp: implementation of the R3DRectangle class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DRectangle.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DRectangle::R3DRectangle()
{
	mTexOffset_s=mTexOffset_t=0;
	mTexScale=1;	// initialise texture settings
}

R3DRectangle::~R3DRectangle()
{

}

/* corners look like this:

  0---------1
  |			|
  |			|
  |			|
  |			|
  2---------3

*/

// set up rectangle using two positions, generate the other two automatically
void R3DRectangle::SetPosition(const R3DVector& v1, const R3DVector& v2)
{
	mCorners[0]=v1;
	mCorners[1].SetPosition(v2.m_vx, v1.m_vy, v2.m_vz);
	mCorners[2].SetPosition(v1.m_vx, v2.m_vy, v1.m_vz);
	mCorners[3]=v2;
}

// set up rectangle from given all for corners
void R3DRectangle::SetPosition(const R3DVector & v1, const R3DVector & v2, const R3DVector & v3, const R3DVector & v4)
{
	mCorners[0].SetPosition(v1);
	mCorners[1].SetPosition(v2);
	mCorners[2].SetPosition(v3);
	mCorners[3].SetPosition(v4);
}


// rotate the rectangle:
void R3DRectangle::RotateX(R3DVector Origin, float Theta)
{
	for(int x=0; x<=3; x++)
		mCorners[x].RotateX(Origin, Theta);
}

void R3DRectangle::RotateY(R3DVector Origin, float Theta)
{
	for(int x=0; x<=3; x++)
		mCorners[x].RotateY(Origin, Theta);
}

void R3DRectangle::RotateZ(R3DVector Origin, float Theta)
{
	for(int x=0; x<=3; x++)
		mCorners[x].RotateZ(Origin, Theta);
}

// move the rectangle
void R3DRectangle::Move(R3DVector v)
{
	for(int x=0; x<=3; x++)
		mCorners[x].Move(v);
}

// set the texture of the rectangle:
void R3DRectangle::SetTexture(R3DTexture t)
{
	mTexture=t;

	R3DVector top=mCorners[0].GetPosition()-mCorners[1].GetPosition();
	R3DVector side=mCorners[0].GetPosition()-mCorners[2].GetPosition();
	float width=top.Length()*mTexScale;
	float height=side.Length()*mTexScale;

	mCorners[0].SetTexPosition(0.f+mTexOffset_s, 0.f+mTexOffset_t);
	mCorners[1].SetTexPosition(width+mTexOffset_s, 0.f+mTexOffset_t);
	mCorners[2].SetTexPosition(0.f+mTexOffset_s, height+mTexOffset_t);
	mCorners[3].SetTexPosition(width+mTexOffset_s, height+mTexOffset_t);
}


//////////////////////////////////////////////////////
// Actually draw the rectangle!
void R3DRectangle::Draw()
{
	// project all the vertexes into screen co-ordinates
	for(int x=0; x<=3; x++)
		mCorners[x].Project();

	if(mCorners[0].IsDrawable()==FALSE ||
	   mCorners[1].IsDrawable()==FALSE ||
	   mCorners[2].IsDrawable()==FALSE ||
	   mCorners[3].IsDrawable()==FALSE)
		return;	// cannot draw this rectangle, its too close or behind us

	// set the texture:
	mTexture.Activate();
	
	// actually draw the two triangles.  If all vertexes are visible, just draw them:
	if(mCorners[0].IsOnScreen()==TRUE &&
	   mCorners[1].IsOnScreen()==TRUE &&
	   mCorners[2].IsOnScreen()==TRUE &&
	   mCorners[3].IsOnScreen()==TRUE)
		{
		grDrawTriangle(mCorners[0].GetVertex(), mCorners[1].GetVertex(), mCorners[2].GetVertex());
		grDrawTriangle(mCorners[2].GetVertex(), mCorners[1].GetVertex(), mCorners[3].GetVertex());
		}
	else if(mCorners[0].IsOnScreen()==TRUE ||	// only some are visible
	   mCorners[1].IsOnScreen()==TRUE ||
	   mCorners[2].IsOnScreen()==TRUE ||
	   mCorners[3].IsOnScreen()==TRUE)
		{	// so use glide's nifty clipping feature:
		guDrawTriangleWithClip(mCorners[0].GetVertex(), mCorners[1].GetVertex(), mCorners[2].GetVertex());
		guDrawTriangleWithClip(mCorners[2].GetVertex(), mCorners[1].GetVertex(), mCorners[3].GetVertex());
		}
}

// sets up the texture settings
void R3DRectangle::SetupTexture(float offset_s, float offset_t, float scale)
{
	mTexOffset_s=offset_s;
	mTexOffset_t=offset_t;
	if(scale!=0)
		mTexScale=scale;
}

// an easy way to paint an entire texture once over a rectangle:
void R3DRectangle::SetTexture(R3DTexture t, float right, float bottom)
{
	mTexture=t;
	mCorners[0].SetTexPosition(mTexOffset_s, mTexOffset_t);
	mCorners[1].SetTexPosition(right, mTexOffset_t);
	mCorners[2].SetTexPosition(mTexOffset_s, bottom);
	mCorners[3].SetTexPosition(right, bottom);
}

// serialize a rectangle
IMPLEMENT_SERIAL( R3DRectangle, CObject, 1 )
void R3DRectangle::Serialize(CArchive & archive)
{
	GrMipMapId_t t;
	if(archive.IsStoring())
		{
		t=mTexture.GetTextureId();
		archive << mTexOffset_s << mTexOffset_t << mTexScale << t;
		}
	else
		{
		archive >> mTexOffset_s >> mTexOffset_t >> mTexScale >> t;
		mTexture=R3DTexture(t);	// construct texture based on saved ID
		}

	for(int x=0; x<=3; x++)
		mCorners[x].Serialize(archive);
}

// copy constructor
R3DRectangle::R3DRectangle(const R3DRectangle & r)
{
	for(int x=0; x<=3; x++)
		mCorners[x]=r.mCorners[x];
	mTexOffset_s=r.mTexOffset_s;
	mTexOffset_t=r.mTexOffset_t;
	mTexScale=r.mTexScale;
	mTexture=r.mTexture;
}

// asiignment operator
R3DRectangle& R3DRectangle::operator =(const R3DRectangle & r)
{
	for(int x=0; x<=3; x++)
		mCorners[x]=r.mCorners[x];
	mTexOffset_s=r.mTexOffset_s;
	mTexOffset_t=r.mTexOffset_t;
	mTexScale=r.mTexScale;
	mTexture=r.mTexture;
	return *this;
}
