// R3DTriangle.cpp: implementation of the R3DTriangle class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DTriangle.h"
#include "R3DVector.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DTriangle::R3DTriangle()
{
	// default texture settings
	mTexOffset_s=mTexOffset_t=0;
	mTexScale=1;

	// create a default triangle:
	mOrigin.SetPosition(0.f, 0.f, 0.f);
	mVertexes[0].SetPosition(0.f, 0.f, 0.f);
	mVertexes[1].SetPosition(10.f, 0.f, 0.f);
	mVertexes[2].SetPosition(0.f, 10.f, 0.f);
}

R3DTriangle::~R3DTriangle()
{

}

// Draws the triangle in all its triangular glory
void R3DTriangle::Draw()
{
	// project all the vertexes into screen co-ordinates
	for(int x=0; x<=2; x++)
		mVertexes[x].Project();
	
	if(mVertexes[0].IsDrawable()==FALSE ||
	   mVertexes[1].IsDrawable()==FALSE ||
	   mVertexes[2].IsDrawable()==FALSE)
		return;	// cannot draw this triangle, its too close or behind us

	// set the texture:
	mTexture.Activate();
	
	if(mVertexes[0].IsOnScreen()==TRUE &&	// if all vertexes are visible...
	   mVertexes[1].IsOnScreen()==TRUE &&
	   mVertexes[2].IsOnScreen()==TRUE)
		{
		// ...actually draw the triangle:
		grDrawTriangle(mVertexes[0].GetVertex(), mVertexes[1].GetVertex(), mVertexes[2].GetVertex());
		}
	else if(mVertexes[0].IsOnScreen()==TRUE ||	// only some vertexes are visible...
	   mVertexes[1].IsOnScreen()==TRUE ||
	   mVertexes[2].IsOnScreen()==TRUE)
		{
		// ... draw the triangle using glide's nifty clipping function:
		guDrawTriangleWithClip(mVertexes[0].GetVertex(), mVertexes[1].GetVertex(), mVertexes[2].GetVertex());
		}
}

// sets the triangle's texture
void R3DTriangle::SetTexture(R3DTexture t)
{
	mTexture=t;
	R3DVector offset(mTexOffset_s, mTexOffset_t, 0);

	mVertexes[0].SetTexPosition(mTexScale*mVertexes[0].GetPosition()+offset);	// scale texture down by 10
	mVertexes[1].SetTexPosition(mTexScale*mVertexes[1].GetPosition()+offset);
	mVertexes[2].SetTexPosition(mTexScale*mVertexes[2].GetPosition()+offset);
}

// sets the position of the triangle and initialises the texture co-ordinates
void R3DTriangle::SetPosition(R3DVector v1, R3DVector v2, R3DVector v3)
{
	mOrigin=v1;
	mVertexes[0].SetPosition(v1);
	mVertexes[1].SetPosition(v2);
	mVertexes[2].SetPosition(v3);
}

// set position of triangle at construction time!
R3DTriangle::R3DTriangle(R3DVector v1, R3DVector v2, R3DVector v3)
{
		SetPosition(v1, v2, v3);	// simply call SetPosition
}

// each of these rotate functions simply rotates each of the three vertexes:
void R3DTriangle::RotateX(R3DVector Origin, float Theta)
{
	for(int x=0; x<=2; x++)
		mVertexes[x].RotateX(Origin, Theta);
}

void R3DTriangle::RotateY(R3DVector Origin, float Theta)
{
	for(int x=0; x<=2; x++)
		mVertexes[x].RotateY(Origin, Theta);
}

void R3DTriangle::RotateZ(R3DVector Origin, float Theta)
{
	for(int x=0; x<=2; x++)
		mVertexes[x].RotateZ(Origin, Theta);
}

// simply moves each of the vertexes in turn
void R3DTriangle::Move(R3DVector v)
{
	for(int x=0; x<=2; x++)
		mVertexes[x].Move(v);
}

// makes it easy to line up textures
void R3DTriangle::SetupTexture(float offset_s, float offset_t, float scale)
{
	mTexOffset_s=offset_s;
	mTexOffset_t=offset_t;
	mTexScale=scale;
}


// Serialize a triangle
IMPLEMENT_SERIAL( R3DTriangle, CObject, 1 )
void R3DTriangle::Serialize(CArchive & archive)
{
	GrMipMapId_t t;
	
	if(archive.IsStoring())
		{
		t=mTexture.GetTextureId();
		archive << mTexOffset_s << mTexOffset_t << mTexScale << t;
		}
	else
		{
		archive >> mTexOffset_t >> mTexOffset_t >> mTexScale >> t;
		mTexture=R3DTexture(t);	// construct texture based on saved ID
		}
	
	mOrigin.Serialize(archive);
	for(int x=0; x<=2; x++)
		mVertexes[x].Serialize(archive);
}

// copy constructor
R3DTriangle::R3DTriangle(const R3DTriangle & t)
{
	mOrigin=t.mOrigin;
	mTexOffset_s=t.mTexOffset_s;
	mTexOffset_t=t.mTexOffset_t;
	mTexScale=t.mTexScale;
	mTexture=t.mTexture;
	for(int x=0; x<=2; x++)
		mVertexes[x]=t.mVertexes[x];
}

// assignment operator
R3DTriangle& R3DTriangle::operator =(const R3DTriangle & t)
{
	mOrigin=t.mOrigin;
	mTexOffset_s=t.mTexOffset_s;
	mTexOffset_t=t.mTexOffset_t;
	mTexScale=t.mTexScale;
	mTexture=t.mTexture;
	for(int x=0; x<=2; x++)
		mVertexes[x]=t.mVertexes[x];
	return *this;
}
