// R3DVector.cpp: implementation of the R3DVector class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DVector.h"
#include "R3DCalc.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DVector::R3DVector()
{
	m_vx=m_vy=m_vz=0;	// initialise to zero
}

R3DVector::~R3DVector()
{

}

R3DVector::R3DVector(float x, float y, float z)	// constructor allows initialisation
{
	m_vx=x;
	m_vy=y;
	m_vz=z;	// assigns 
}

/////////////////////////////////////////////
// Operators
/////////////////////////////////////////////

void R3DVector::operator +=(R3DVector V)
{
	m_vx+=V.m_vx;
	m_vy+=V.m_vy;
	m_vz+=V.m_vz;	// implied this
}

void R3DVector::operator -=(R3DVector V)
{
	m_vx-=V.m_vx;
	m_vy-=V.m_vy;
	m_vz-=V.m_vz;
}


///////////////////////////////////////////////
// Other Functions
///////////////////////////////////////////////

float R3DVector::Length()
{
	return sqrt(m_vx*m_vx + m_vy*m_vy + m_vz*m_vz);
}




////////////////////////////////////////////////////////////////////////////////
// Global functions that operate on vectors.
// These are not members of R3DVector because they return new R3DVectors
////////////////////////////////////////////////////////////////////////////////

// computes the dot product of two vectors
float operator *(R3DVector A, R3DVector B)
	{
	float Q;	// to store the result

	Q=A.m_vx*B.m_vx;
	Q+=A.m_vy*B.m_vy;
	Q+=A.m_vz*B.m_vz;	// add up the products of each of the axes

	return Q;
	}

// computes the product of a float and a vector
R3DVector operator *(float A, R3DVector B)
	{
	R3DVector Q;	// to store the result

	Q.m_vx=A*B.m_vx;
	Q.m_vy=A*B.m_vy;
	Q.m_vz=A*B.m_vy;

	return Q;
	}

//computes the difference between two vectors
R3DVector operator -(R3DVector A, R3DVector B)
	{
	R3DVector Q;	// to store the result

	Q.m_vx=A.m_vx-B.m_vx;
	Q.m_vy=A.m_vy-B.m_vy;
	Q.m_vz=A.m_vz-B.m_vz;

	return Q;
	}

// computes the negative of a vector
R3DVector operator -(R3DVector A)	// unary version
	{
	R3DVector Q;	// to store the result

	Q.m_vx= -A.m_vx;
	Q.m_vy= -A.m_vy;
	Q.m_vz= -A.m_vz;

	return Q;
	}

//  computes the sum of two vectors
R3DVector operator +(R3DVector A, R3DVector B)
	{
	R3DVector Q;

	Q.m_vx=A.m_vx+B.m_vx;
	Q.m_vy=A.m_vy+B.m_vy;
	Q.m_vz=A.m_vz+B.m_vz;

	return Q;
	}

// computes the angle between two vectors
float operator ^(R3DVector A, R3DVector B)
	{
	float TempA, TempB;

	TempA=A*B;						// dot product
	TempB=A.Length()*B.Length();	// product of modules

	return acos(TempA/TempB);
	}


// computes the cross product
R3DVector XProduct(R3DVector A, R3DVector B)
	{
	R3DVector Q;	// to store the result

	Q.m_vx=A.m_vy*B.m_vz - A.m_vz*B.m_vy;
	Q.m_vy=A.m_vz*B.m_vx - A.m_vx*B.m_vy;
	Q.m_vz=A.m_vx*B.m_vy - A.m_vy*B.m_vz;

	return Q;
	}

// an easy way to set the position of a vector
void R3DVector::SetPosition(float x, float y, float z)
{
	m_vx=x;
	m_vy=y;
	m_vz=z;
}


// rotate vector about a specified point by a specified angle:
///////////////////////////////////////////////////////////////////

void R3DVector::RotateX(R3DVector Origin, float Theta)
{
	float ny, nz, sintheta, costheta;
	
	R3DVector rp= *this - Origin;	// position relative to rotation origin

	costheta=R3DCALC->Cos(Theta);
	sintheta=R3DCALC->Sin(Theta);

	ny=(rp.m_vy*costheta)-(rp.m_vz*sintheta);
	nz=(rp.m_vy*sintheta)+(rp.m_vz*costheta);	// generate the new positions

	this->m_vy=ny+Origin.m_vy;
	this->m_vz=nz+Origin.m_vz;	// position relative to world origin
}

void R3DVector::RotateY(R3DVector Origin, float Theta)
{
	float nx, nz, sintheta, costheta;
	
	R3DVector rp= *this - Origin;	// position relative to rotation origin
	
	costheta=R3DCALC->Cos(Theta);
	sintheta=R3DCALC->Sin(Theta);

	nx=(rp.m_vx*costheta)-(rp.m_vz*sintheta);
	nz=(rp.m_vx*sintheta)+(rp.m_vz*costheta);	// generate the new positions

	this->m_vx=nx+Origin.m_vx;
	this->m_vz=nz+Origin.m_vz;	// position relative to world origin
}

void R3DVector::RotateZ(R3DVector Origin, float Theta)
{
	float nx, ny, sintheta, costheta;
	
	R3DVector rp= *this - Origin;	// position relative to rotation origin

	costheta=R3DCALC->Cos(Theta);
	sintheta=R3DCALC->Sin(Theta);

	nx=(rp.m_vx*costheta)-(rp.m_vy*sintheta);
	ny=(rp.m_vx*sintheta)+(rp.m_vy*costheta);	// generate the new positions

	this->m_vy=ny+Origin.m_vy;
	this->m_vx=nx+Origin.m_vx;	// position relative to world origin
}

// serialize a vector
IMPLEMENT_SERIAL( R3DVector, CObject, 1 )
void R3DVector::Serialize(CArchive & archive)
{
	if(archive.IsStoring())
		archive << m_vx << m_vy << m_vz;
	else
		archive >> m_vx >> m_vy >> m_vz;
}
