// R3DVertex.cpp: implementation of the R3DVertex class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Traffic3D.h"
#include "R3DVertex.h"
#include "R3DCamera.h"
#include <math.h>	// for sin and cosine 

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DVertex::R3DVertex()
{
	ms=mt=0.f;
	mGVertex.r=mGVertex.g=mGVertex.b=128.f;
	mGVertex.a=0.f;	// set default colour and alpha
	mTransformed=TRUE;
}

R3DVertex::~R3DVertex()
{

}

// calculates the screen co-ordinates based on the world co-ordinates and updates the
// relevant GrVertex members
void R3DVertex::Project()
	{
	R3DCamera *rcam=R3DCAM;	// point to the current camera
	
	if(mTransformed || rcam->IsTransformed())	// if vertex or camera have changed
		{
		//R3DVector camera(320.f, 240.f, -640.f);	// for now assume eye is at (320, 240, -640)
		R3DVector cam2pt=mWorldOrigin;	//-camera;	// position relative to camera
		R3DVector cam2spt;	// screen position

		// First apply camera transformation:
		cam2pt=rcam->VProject(cam2pt);
		if(cam2pt.m_vz<1)	// if too close or behind camera
			{
			mDrawable=FALSE;
			return;	// no point continuing as this vertex should NEVER be sent to the card
			}
		else
			{
			mDrawable=TRUE;
			}
	
		//cam2spt.m_vz=0-camera.m_vz;	// screen point is on z axis
		cam2spt.m_vx=(cam2pt.m_vx/cam2pt.m_vz)*640;	// plot intersection with z axis
		cam2spt.m_vy=(cam2pt.m_vy/cam2pt.m_vz)*640;	// and again for the y's
		// note:  Focal length=640
	
		// simply add 320, 240 to place centre of screen at 0, 0
		mScreenOrigin.m_vx=cam2spt.m_vx+320.f;	//+camera.m_vx;
		mScreenOrigin.m_vy=cam2spt.m_vy+240.f;	//+camera.m_vy;
		// leave the z value alone as its not really used:
		mScreenOrigin.m_vz=mWorldOrigin.m_vz;
	
		// now construct the glide GrVertex based on the screen co-ordinates:
		mGVertex.x=mScreenOrigin.m_vx;
		mGVertex.y=mScreenOrigin.m_vy;
		mGVertex.z=mScreenOrigin.m_vz;
		// oow is 1/distance from eye to object:
		float oow=1.f/cam2pt.m_vz;
		mGVertex.oow=oow;
		mGVertex.tmuvtx[0].sow=oow*ms;
		mGVertex.tmuvtx[0].tow=oow*mt;	// sow & tow based on texture s and t co-ords
		// this would be a good time to snap the vertices:
		const float vertex_snapper = ( float ) ( 3L << 18 );
		mGVertex.x+=vertex_snapper;
		mGVertex.x-=vertex_snapper;
		mGVertex.y+=vertex_snapper;
		mGVertex.y-=vertex_snapper;

		// if all 3 vertexes in a triangle are off the screen it isn't drawn...
		if(mScreenOrigin.m_vx<0 || mScreenOrigin.m_vx>640 ||
		   mScreenOrigin.m_vy<0 || mScreenOrigin.m_vy>480)
			{
			mOnScreen=FALSE;
			}
		else
			{
			mOnScreen=TRUE;
			}
		}
	mTransformed=FALSE;	// most of the above doesn't have to be redone
						// unless the camera or the vertex are transformed
	}

// constructor sets up vertex at specified position:
R3DVertex::R3DVertex(R3DVector v)
{
	ms=mt=0.f;
	mWorldOrigin=v;	// set position
	mTransformed=TRUE;
}

// rotates the vertex about the X axis
void R3DVertex::RotateX(R3DVector Origin, float Theta)
{
	mWorldOrigin.RotateX(Origin, Theta);
	mTransformed=TRUE;
}

// rotates vertex about the Y axis
void R3DVertex::RotateY(R3DVector Origin, float Theta)
{
	mWorldOrigin.RotateY(Origin, Theta);
	mTransformed=TRUE;
}

// rotates vertex about the z axis
void R3DVertex::RotateZ(R3DVector Origin, float Theta)
{
	mWorldOrigin.RotateZ(Origin, Theta);
	mTransformed=TRUE;
}

// sets the position of the vertex
void R3DVertex::SetPosition(R3DVector p)
{
	mWorldOrigin=p;
	mTransformed=TRUE;
}

// sets the colour of the vertex
void R3DVertex::SetColour(R3DVector c)
{
	// x, y and z of the vector map to r, g and b of the colour respectively
	mGVertex.r=c.m_vx;
	mGVertex.g=c.m_vy;
	mGVertex.b=c.m_vz;
	mGVertex.a=0;	// no alpha for now
}

// an easy way to set the position of a vertex
void R3DVertex::SetPosition(float x, float y, float z)
{
	mWorldOrigin.m_vx=x;
	mWorldOrigin.m_vy=y;
	mWorldOrigin.m_vz=z;
	mTransformed=TRUE;
}

// projects the appropriate texture onto the vertex
void R3DVertex::SetTexPosition(R3DVector v)
{
	ms=v.m_vx;
	mt=v.m_vy;
	// NOTE:  later, some kind of projection will have to be done to take into account
	// the z co-ordinate.
}

// moves the vertex by the distance specified
void R3DVertex::Move(R3DVector v)
	{
	mWorldOrigin+=v;
	mTransformed=TRUE;
	}


// sets texture position with floats instead of a vector
void R3DVertex::SetTexPosition(float x, float y)
{
	ms=x;
	mt=y;
}

// serialize a vertex
IMPLEMENT_SERIAL( R3DVertex, CObject, 1 )
void R3DVertex::Serialize(CArchive & archive)
{
	if(archive.IsStoring())
		{
		archive.Write(&mGVertex, sizeof(GrVertex));
		archive << ms << mt;
		}
	else
		{
		archive.Read(&mGVertex, sizeof(GrVertex));
		archive >> ms >> mt;
		}
	mScreenOrigin.Serialize(archive);
	mWorldOrigin.Serialize(archive);
}

// assignment operator
R3DVertex& R3DVertex::operator =(const R3DVertex & v)
{
	//memcpy(mGVertex, v.mGVertex, sizeof(GrVertex));
	mGVertex=v.mGVertex;
	ms=v.ms;
	mt=v.mt;
	mScreenOrigin=v.mScreenOrigin;
	mWorldOrigin=v.mWorldOrigin;
	return *this;
}
