// R3DfxHardware.cpp: implementation of the R3DfxHardware class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "Cube3D.h"
#include "R3DfxHardware.h"
#include "R3DCamera.h"
#include "R3DCalc.h"
#include <glide.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// globals:
R3DfxHardware *R3DHW;	// a global pointer to the hardware device
static R3DCamera globalcam;
static R3DCalc globalcalc;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

R3DfxHardware::R3DfxHardware()
{
	Initialised=FALSE;
}

R3DfxHardware::~R3DfxHardware()
{

}

int R3DfxHardware::InitHardware()
{
	// *********** Initialise 3Dfx hardware ***********************
	Initialised=FALSE;
	grGlideInit();	// start glide

	// set up the global pointers to static objects:
	R3DHW=this;	// set up the global hardware pointer
	R3DCAM=&globalcam;
	R3DCALC=&globalcalc;
	
	if (grSstQueryHardware(&hwconfig)==FXTRUE)
		{			
			// Select Sst(0) only (Useful if more than one card present)
			grSstSelect(0); 
	
			//Basic screen drawing options
			//Using NULL for the window handle (1st parameter below) only works on Voodoo cards
			grSstWinOpen(NULL, GR_RESOLUTION_640x480, GR_REFRESH_60Hz, GR_COLORFORMAT_RGBA, GR_ORIGIN_UPPER_LEFT, 2, 0); 
			Initialised=TRUE;
			SetDefaultOptions();
			grBufferClear(0, 0, GR_WDEPTHVALUE_FARTHEST); // clear the buffers
			grBufferSwap(1);	// swap buffers

			//grCullMode(GR_CULL_NEGATIVE);	// turn on back face culling
			mCurrentTexture=999;
			return 1;
		}
	
	else
		{
		//Bomb out gracefully
			AfxMessageBox("Error:  could not open hardware.\n");
			return 0;
		}
}

void R3DfxHardware::SetDefaultOptions()
	{
	grDepthBufferMode( GR_DEPTHBUFFER_WBUFFER );	// select W-buffering
	grDepthBufferFunction( GR_CMP_LESS );			// bigger W's = further away
	grDepthMask( FXTRUE );							// switch on depth buffering
	// switches off texture mapping but combined with iterated rgb
	grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL,
					GR_COMBINE_LOCAL_ITERATED,
					GR_COMBINE_OTHER_NONE, FXFALSE);
		
	// slap on textures
	grTexCombineFunction(GR_TMU0, GR_TEXTURECOMBINE_DECAL);
	}


void R3DfxHardware::CloseHardware()
{
	if (Initialised)
		grGlideShutdown();
	Initialised=FALSE;
}

GrMipMapId_t R3DfxHardware::LoadTexture(char * filename)
{
	// Credit to andrew.hodgson1@virgin.net for this function
	GrMipMapId_t MyDecal;
	GrMipMapId_t *decal=&MyDecal;
	Gu3dfInfo info;

	//file does not exist
	if ( !gu3dfGetInfo( filename , &info ) )
		{
		grGlideShutdown();
		AfxMessageBox("Error:  could not load texture.\n");
		}
	else
		{
		//allocate temporary system memory
		info.data = malloc( info.mem_required );

		//not enough free memory
		if ( info.data == 0 )
			{
			printf( "Not enough memory for texture\n" );
			grGlideShutdown();
			exit( -1 );
			}

		//buggered file :)
		if ( !gu3dfLoad( filename , &info ) )
			{
			printf( "Cannot load texture file\n" );
			grGlideShutdown();
			exit( -1 );
			}

		//load texture, get handle , set bilinear filtering, etc
		*decal = guTexAllocateMemory( 0, GR_MIPMAPLEVELMASK_BOTH,
		//texture dimensions
		info.header.width, info.header.height,
		info.header.format,
		//mipmapping mode
		GR_MIPMAP_NEAREST,
		//LOD settings & aspect ratios for mipmaps
		info.header.small_lod, info.header.large_lod,
		info.header.aspect_ratio,
		//wrap around x & y
		GR_TEXTURECLAMP_WRAP, GR_TEXTURECLAMP_WRAP,
		//use bilinear filtering
		GR_TEXTUREFILTER_BILINEAR, GR_TEXTUREFILTER_BILINEAR,
		0.0F,
		FXFALSE );

		//card is out of texture memory
		if ( *decal == GR_NULL_MIPMAP_HANDLE )
			{
			printf( "could not allocate memory for Texture\n" );
			grGlideShutdown();
			exit( -1 );
			}

		//send the map to the card
		guTexDownloadMipMap( *decal, info.data, &info.table.nccTable );
		//free system memory (no longer required now texture is on card
		free( info.data );
		}
	return MyDecal;
}

void R3DfxHardware::ActivateTextures()
{
	// combine iterated RGB with texture mapping
	grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL,
								GR_COMBINE_LOCAL_ITERATED,
								GR_COMBINE_OTHER_TEXTURE, FXFALSE);
}


// simply calls the appropriate glide function to display the next frame of animation
void R3DfxHardware::BufferSwap()
{
	grBufferSwap(0);	// waits for the next vertical retrace
}

// clears the current buffer ready to begin rendering the next frame
void R3DfxHardware::BufferClear()
{
	grBufferClear(0, 0, GR_WDEPTHVALUE_FARTHEST);	// clears to black
}


// sets a new texture only if it is different from the one currently in use
void R3DfxHardware::SetTexture(GrMipMapId_t tex)
{
	if(tex!=mCurrentTexture)
		{
		guTexSource(tex);
		mCurrentTexture=tex;
		}
}
